@@ -29,10 +29,10 @@ public:
     }
 
 #if wxUSE_STREAMS
-    virtual bool LoadFile(wxImage *image, wxInputStream& stream, bool verbose=true, int index=-1) wxOVERRIDE;
-    virtual bool SaveFile(wxImage *image, wxOutputStream& stream, bool verbose=true) wxOVERRIDE;
+    virtual bool LoadFile(wxImage *image, wxInputStream& stream, bool verbose=true, int index=-1) override;
+    virtual bool SaveFile(wxImage *image, wxOutputStream& stream, bool verbose=true) override;
 protected:
-    virtual bool DoCanRead(wxInputStream& stream) wxOVERRIDE;
+    virtual bool DoCanRead(wxInputStream& stream) override;
 #endif
 
     wxDECLARE_DYNAMIC_CLASS(wxIFFHandler);
