@@ -48,11 +48,11 @@ public:
 
     // Override base class method to not do anything but always return success:
     // we don't need this as we do our validation on the fly here.
-    virtual bool Validate(wxWindow * WXUNUSED(parent)) wxOVERRIDE { return true; }
+    virtual bool Validate(wxWindow * WXUNUSED(parent)) override { return true; }
 
     // Override base class method to check that the window is a text control or
     // combobox.
-    virtual void SetWindow(wxWindow *win) wxOVERRIDE;
+    virtual void SetWindow(wxWindow *win) override;
 
 protected:
     wxNumValidatorBase(int style)
@@ -71,7 +71,7 @@ protected:
     }
 
     // Get the text entry of the associated control. Normally shouldn't ever
-    // return NULL (and will assert if it does return it) but the caller should
+    // return nullptr (and will assert if it does return it) but the caller should
     // still test the return value for safety.
     wxTextEntry *GetTextEntry() const;
 
@@ -190,7 +190,7 @@ public:
         max = GetMax();
     }
 
-    virtual bool TransferToWindow()  wxOVERRIDE
+    virtual bool TransferToWindow()  override
     {
         if ( m_value )
         {
@@ -204,7 +204,7 @@ public:
         return true;
     }
 
-    virtual bool TransferFromWindow() wxOVERRIDE
+    virtual bool TransferFromWindow() override
     {
         if ( m_value )
         {
@@ -237,8 +237,13 @@ protected:
 
     // Implement wxNumValidatorBase virtual method which is the same for
     // both integer and floating point numbers.
-    virtual wxString NormalizeString(const wxString& s) const wxOVERRIDE
+    virtual wxString NormalizeString(const wxString& s) const override
     {
+        // If empty value is handled as zero, just return it as is for
+        // consistency with TransferFromWindow() and NormalizeValue().
+        if ( s.empty() && BaseValidator::HasFlag(wxNUM_VAL_ZERO_AS_BLANK) )
+            return wxString();
+
         LongestValueType value;
         if ( !BaseValidator::FromString(s, &value) )
         {
@@ -259,7 +264,7 @@ protected:
         return NormalizeValue(value);
     }
 
-    virtual bool CanBeNegative() const wxOVERRIDE { return m_min < 0; }
+    virtual bool CanBeNegative() const override { return m_min < 0; }
 
 
     // This member is protected because it can be useful to the derived classes
@@ -288,7 +293,7 @@ private:
     // Minimal and maximal values accepted (inclusive).
     ValueType m_min, m_max;
 
-    wxDECLARE_NO_ASSIGN_CLASS(wxNumValidator);
+    wxDECLARE_NO_ASSIGN_DEF_COPY(wxNumValidator);
 };
 
 } // namespace wxPrivate
@@ -307,13 +312,8 @@ protected:
     // Define the type we use here, it should be the maximal-sized integer type
     // we support to make it possible to base wxIntegerValidator<> for any type
     // on it.
-#ifdef wxLongLong_t
     typedef wxLongLong_t LongestValueType;
     typedef wxULongLong_t ULongestValueType;
-#else
-    typedef long LongestValueType;
-    typedef unsigned long ULongestValueType;
-#endif
 
     wxIntegerValidatorBase(int style)
         : wxNumValidatorBase(style)
@@ -330,11 +330,20 @@ protected:
 
     virtual bool IsInRange(LongestValueType value) const = 0;
 
+    // Check whether the value is in the extended range allowed on input.
+    //
+    // Notice that the minimal and maximal values that have to be accepted on
+    // input may differ from the actually defined range to allow entering
+    // a temporarily invalid number because otherwise it would not be possible
+    // to enter any digits in an initially empty control limited to the values
+    // between "10" and "20".
+    virtual bool IsInInputRange(LongestValueType value) const = 0;
+
     // Implement wxNumValidatorBase pure virtual method.
-    virtual bool IsCharOk(const wxString& val, int pos, wxChar ch) const wxOVERRIDE;
+    virtual bool IsCharOk(const wxString& val, int pos, wxChar ch) const override;
 
 private:
-    wxDECLARE_NO_ASSIGN_CLASS(wxIntegerValidatorBase);
+    wxDECLARE_NO_ASSIGN_DEF_COPY(wxIntegerValidatorBase);
 };
 
 // Validator for integer numbers. It can actually work with any integer type
@@ -356,7 +365,7 @@ public:
     //
     // Sets the range appropriately for the type, including setting 0 as the
     // minimal value for the unsigned types.
-    wxIntegerValidator(ValueType *value = NULL, int style = wxNUM_VAL_DEFAULT)
+    wxIntegerValidator(ValueType *value = nullptr, int style = wxNUM_VAL_DEFAULT)
         : Base(value, style)
     {
         this->SetMin(std::numeric_limits<ValueType>::min());
@@ -376,9 +385,23 @@ public:
     this->SetMax(max);
   }
 
-    virtual wxObject *Clone() const wxOVERRIDE { return new wxIntegerValidator(*this); }
+    wxNODISCARD virtual wxObject *Clone() const override { return new wxIntegerValidator(*this); }
 
-    virtual bool IsInRange(LongestValueType value) const wxOVERRIDE
+    virtual bool IsInRange(LongestValueType value) const override
+    {
+        return IsValueInRange( value, this->GetMin(), this->GetMax() );
+    }
+
+    virtual bool IsInInputRange(LongestValueType value) const override
+    {
+        ValueType min = wxMin( 1, this->GetMin() );
+        ValueType max = wxMax( 0, this->GetMax() );
+
+        return IsValueInRange( value, min, max );
+    }
+
+private:
+    bool IsValueInRange(LongestValueType value, ValueType min, ValueType max ) const
     {
         // LongestValueType is used as a container for the values of any type
         // which can be used in type-independent wxIntegerValidatorBase code,
@@ -393,11 +416,10 @@ public:
             return false;
         }
 
-        return this->GetMin() <= valueT && valueT <= this->GetMax();
+        return min <= valueT && valueT <= max;
     }
 
-private:
-    wxDECLARE_NO_ASSIGN_CLASS(wxIntegerValidator);
+    wxDECLARE_NO_ASSIGN_DEF_COPY(wxIntegerValidator);
 };
 
 // Helper function for creating integer validators which allows to avoid
@@ -448,8 +470,17 @@ protected:
 
     virtual bool IsInRange(LongestValueType value) const = 0;
 
+    // Check whether the value is in the extended range allowed on input.
+    //
+    // Notice that the minimal and maximal values that have to be accepted on
+    // input may differ from the actually defined range to allow entering
+    // a temporarily invalid number because otherwise it would not be possible
+    // to enter any digits in an initially empty control limited to the values
+    // between "10" and "20".
+    virtual bool IsInInputRange(LongestValueType value) const = 0;
+
     // Implement wxNumValidatorBase pure virtual method.
-    virtual bool IsCharOk(const wxString& val, int pos, wxChar ch) const wxOVERRIDE;
+    virtual bool IsCharOk(const wxString& val, int pos, wxChar ch) const override;
 
 private:
     // Maximum number of decimals digits after the decimal separator.
@@ -458,7 +489,7 @@ private:
     // Factor applied for the displayed the value.
     double m_factor;
 
-    wxDECLARE_NO_ASSIGN_CLASS(wxFloatingPointValidatorBase);
+    wxDECLARE_NO_ASSIGN_DEF_COPY(wxFloatingPointValidatorBase);
 };
 
 // Validator for floating point numbers. It can be used with float, double or
@@ -474,7 +505,7 @@ public:
 
 
     // Ctor using implicit (maximal) precision for this type.
-    wxFloatingPointValidator(ValueType *value = NULL,
+    wxFloatingPointValidator(ValueType *value = nullptr,
                              int style = wxNUM_VAL_DEFAULT)
         : Base(value, style)
     {
@@ -485,7 +516,7 @@ public:
 
     // Ctor specifying an explicit precision.
     wxFloatingPointValidator(int precision,
-                      ValueType *value = NULL,
+                      ValueType *value = nullptr,
                       int style = wxNUM_VAL_DEFAULT)
         : Base(value, style)
     {
@@ -494,18 +525,28 @@ public:
         this->SetPrecision(precision);
     }
 
-    virtual wxObject *Clone() const wxOVERRIDE
+    wxNODISCARD virtual wxObject *Clone() const override
     {
         return new wxFloatingPointValidator(*this);
     }
 
-    virtual bool IsInRange(LongestValueType value) const wxOVERRIDE
+    virtual bool IsInRange(LongestValueType value) const override
     {
         const ValueType valueT = static_cast<ValueType>(value);
 
         return this->GetMin() <= valueT && valueT <= this->GetMax();
     }
 
+    virtual bool IsInInputRange(LongestValueType value) const override
+    {
+        const ValueType valueT = static_cast<ValueType>(value);
+
+        ValueType min = wxMin( 0, this->GetMin() );
+        ValueType max = wxMax( 0, this->GetMax() );
+
+        return min <= valueT && valueT <= max;
+    }
+
 private:
     void DoSetMinMax()
     {
